# encoding: utf-8
# frozen_string_literal: true

require "zip"

def convert_croms(croms)
    sprites = String.new
    croms.map(&:bytes).each_slice(2) do |(odd,even)|
        (odd.size/64).times do |tile|
            16.times do |y|
                baseoffset = tile*64 + y*2
                bp0 = odd[baseoffset+32] + (odd[baseoffset+0]<<8)
                bp1 = odd[baseoffset+33] + (odd[baseoffset+1]<<8)
                bp2 = even[baseoffset+32] + (even[baseoffset+0]<<8)
                bp3 = even[baseoffset+33] + (even[baseoffset+1]<<8)
                8.times do |x|
                    #p bp0,bp1,bp2,bp3
                    sprites << (bp0[x*2  ]* 1+bp1[x*2  ]* 2+bp2[x*2  ]* 4+bp3[x*2  ]*  8 +
                                bp0[x*2+1]*16+bp1[x*2+1]*32+bp2[x*2+1]*64+bp3[x*2+1]*128)
                end
            end
        end
    end
    return sprites
end

def convert_srom(srom)
    tiles = String.new
    srom.bytes.each_slice(32) do |data|
        8.times do |y|
            4.times do |x|
                tiles << data[y+(x^2)*8]
            end
        end
    end
    return tiles
end

case ARGV[0]
when "mslug"
    Zip::File.open("roms/mslug.zip") do |zip|
        File.binwrite("mslug.crom",convert_croms(%w[1 2 3 4].map{|n|zip.read("201-c#{n}.bin")}))
        File.binwrite("mslug.srom",convert_srom(zip.read("201-s1.bin")))
    end
when "twinspri"
    Zip::File.open("roms/twinspri.zip") do |zip|
        File.binwrite("twinspri.crom",convert_croms(%w[1 2 3 4].map{|n|zip.read("224-c#{n}.bin")}))
        File.binwrite("twinspri.srom",convert_srom(zip.read("224-s1.bin")))
    end
when "samsho"
    Zip::File.open("roms/samsho.zip") do |zip|
        File.binwrite("samsho.crom",convert_croms(%w[1 2 3 4 51 61].map{|n|zip.read("045-c#{n}.bin")}))
        File.binwrite("samsho.srom",convert_srom(zip.read("045-s1.bin")))
    end
else
    puts "idk wtf"
end